declare namespace _default {
    export const POSTRENDER: string;
    export const MOVESTART: string;
    export const MOVEEND: string;
}
export default _default;
/**
 * *
 */
export type Types = "postrender" | "movestart" | "moveend";
//# sourceMappingURL=MapEventType.d.ts.map