export default GML32;
/**
 * @classdesc Feature format for reading and writing data in the GML format
 *            version 3.2.1.
 * @api
 */
declare class GML32 extends GML3 {
    /**
     * @param {import("./GMLBase.js").Options} [opt_options] Optional configuration object.
     */
    constructor(opt_options?: import("./GMLBase.js").Options | undefined);
}
import GML3 from "./GML3.js";
//# sourceMappingURL=GML32.d.ts.map