declare namespace _default {
    export const LAYERGROUP: string;
    export const SIZE: string;
    export const TARGET: string;
    export const VIEW: string;
}
export default _default;
//# sourceMappingURL=MapProperty.d.ts.map