declare namespace _default {
    export const PRERENDER: string;
    export const POSTRENDER: string;
    export const PRECOMPOSE: string;
    export const POSTCOMPOSE: string;
    export const RENDERCOMPLETE: string;
}
export default _default;
export type MapRenderEventTypes = "postrender" | "precompose" | "postcompose" | "rendercomplete";
export type LayerRenderEventTypes = "prerender" | "postrender";
//# sourceMappingURL=EventType.d.ts.map