declare namespace _default {
    export const OPACITY: string;
    export const VISIBLE: string;
    export const EXTENT: string;
    export const Z_INDEX: string;
    export const MAX_RESOLUTION: string;
    export const MIN_RESOLUTION: string;
    export const MAX_ZOOM: string;
    export const MIN_ZOOM: string;
    export const SOURCE: string;
}
export default _default;
//# sourceMappingURL=Property.d.ts.map