/**
 * @module ol/source/common
 */
/**
 * Default WMS version.
 * @type {string}
 */
export const DEFAULT_WMS_VERSION: string;
/**
 * Context options to disable image smoothing.
 * @type {Object}
 */
export const IMAGE_SMOOTHING_DISABLED: Object;
//# sourceMappingURL=common.d.ts.map