declare namespace _default {
    export const ARRAY_BUFFER: string;
    export const JSON: string;
    export const TEXT: string;
    export const XML: string;
}
export default _default;
//# sourceMappingURL=FormatType.d.ts.map