declare namespace _default {
    export const XY: string;
    export const XYZ: string;
    export const XYM: string;
    export const XYZM: string;
}
export default _default;
//# sourceMappingURL=GeometryLayout.d.ts.map