declare namespace _default {
    export const UNDEFINED: string;
    export const LOADING: string;
    export const READY: string;
    export const ERROR: string;
}
export default _default;
//# sourceMappingURL=State.d.ts.map