declare namespace _default {
    export const CIRCLE: string;
    export const DEFAULT: string;
    export const IMAGE: string;
    export const LINE_STRING: string;
    export const POLYGON: string;
    export const TEXT: string;
}
export default _default;
//# sourceMappingURL=BuilderType.d.ts.map