declare namespace _default {
    export const ADDFEATURE: string;
    export const CHANGEFEATURE: string;
    export const CLEAR: string;
    export const REMOVEFEATURE: string;
    export const FEATURESLOADSTART: string;
    export const FEATURESLOADEND: string;
    export const FEATURESLOADERROR: string;
}
export default _default;
export type VectorSourceEventTypes = "clear" | "addfeature" | "changefeature" | "removefeature" | "featuresloadstart" | "featuresloadend" | "featuresloaderror";
//# sourceMappingURL=VectorEventType.d.ts.map