declare namespace _default {
    export const CENTER: string;
    export const RESOLUTION: string;
    export const ROTATION: string;
}
export default _default;
//# sourceMappingURL=ViewProperty.d.ts.map