declare namespace _default {
    export const BOTTOM_LEFT: string;
    export const BOTTOM_CENTER: string;
    export const BOTTOM_RIGHT: string;
    export const CENTER_LEFT: string;
    export const CENTER_CENTER: string;
    export const CENTER_RIGHT: string;
    export const TOP_LEFT: string;
    export const TOP_CENTER: string;
    export const TOP_RIGHT: string;
}
export default _default;
//# sourceMappingURL=OverlayPositioning.d.ts.map