declare namespace _default {
    export const SINGLECLICK: string;
    export const CLICK: string;
    export const DBLCLICK: string;
    export const POINTERDRAG: string;
    export const POINTERMOVE: string;
    export const POINTERDOWN: string;
    export const POINTERUP: string;
    export const POINTEROVER: string;
    export const POINTEROUT: string;
    export const POINTERENTER: string;
    export const POINTERLEAVE: string;
    export const POINTERCANCEL: string;
}
export default _default;
/**
 * *
 */
export type Types = "click" | "dblclick" | "pointermove" | "singleclick" | "pointerdrag";
//# sourceMappingURL=MapBrowserEventType.d.ts.map